<?php
/**
 * Class for work with Postlinks API
 *
 * @method Authenticate_Check(array $params)
 * @method GetAccountInfo(array $params)
 * @method Logout(array $params)
 * @method GetCategories(array $params)
 * @method GetDeniedWords(array $params)
 * @method GetArticleProfiles(array $params)
 * @method ArticleProfile_Create(array $params)
 * @method ArticleProfile_Edit(array $params)
 * @method ArticleProfile_Delete(array $params)
 * @method GetArticles(array $params)
 * @method Article_Create(array $params)
 * @method Article_Edit(array $params)
 * @method Article_Delete(array $params)
 * @method GetPosts(array $params)
 * @method GetPostByUrl(array $params)
 * @method DeletePostByUrl(array $params)
 * @method Post_Delete(array $params)
 * @method GetQueuedPosts(array $params)
 * @method QueuedPosts_Start(array $params)
 * @method QueuedPosts_Stop(array $params)
 * @method QueuedPosts_Delete(array $params)
 * @method Article_QueuePosts(array $params)
 * @method GetLinkProfiles(array $params)
 * @method LinkProfile_Create(array $params)
 * @method LinkProfile_Edit(array $params)
 * @method LinkProfile_Delete(array $params)
 * @method GetCommentLinks(array $params)
 * @method GetContextualLinks(array $params)
 * @method GetExistLinks(array $params)
 * @method CommentLink_Delete(array $params)
 * @method ContextualLink_Delete(array $params)
 * @method GetQueuedLinks(array $params)
 * @method QueuedLinks_Start(array $params)
 * @method QueuedLinks_Stop(array $params)
 * @method QueuedLinks_Delete(array $params)
 * @method QueuedLinks_SetNotUpdated(array $params)
 * @method QueuedLinks_Update(array $params)
 * @method Comment_QueueLinks(array $params)
 * @method Context_QueueLinks(array $params)
 * @method GetFolders(array $params)
 * @method Folder_Create(array $params)
 * @method Folder_Edit(array $params)
 * @method Folder_Delete(array $params)
 * @method Nofollow_Link(array $params)
 * @method Nofollow_Post(array $params)
 * @method GetCountInventoryArticle(array $params)
 *
 */
class PostLinksJson_API
{
	// default wsdl url
	private $_api_url = 'http://postlinks.com/api/json/v1.1/postlinks_json.php';
	
	// Current authentication key. May be restored.
	private $_auth_key;
	
	/** @var Exception $_exception */
	private $_exception = null;

	static private $instance = null;

	/**
	 */
	private function __construct()
	{

	}

	/**
	 * Singleton Pattern
	 * 
     * @return null|PostLinksJson_API
	 */
	static function getInstance()
	{
		if (self::$instance == null)
		{
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __clone()
	{
	}
	
	/**
	 * Magic method
	 *
	 * @param $name string function name
	 * @param $arguments array function arguments
	 * @return mixed
	 */
	public function __call($name, $arguments)
    {
		return $this->execute($name, $arguments[0]);
    }

	
	/**
	 * Check login and password and set the authentication key
	 *
	 * @param $sLogin string
	 * @param $sPassword string
	 * @return boolean
	 */
	function Authenticate($sLogin, $sPassword)
	{
		$this->_auth_key = $this->execute('Authenticate', array('sLogin'=>$sLogin, 'sPassword'=>md5($sPassword)));
		return ($this->_auth_key != '');
	}
	
	/**
	 * Restore authentication key
	 * Check authentication key is not expired and restore it
	 *
	 * @param $auth_key string
	 * @return boolean
	 */
	function AuthenticateRestore($auth_key)
	{
		// @todo check auth_key
		if ($this->execute('Authenticate_Check', array('auth_key' => $auth_key)) === true)
		{
			$this->_auth_key = $auth_key;
			return true;
		}
		else
		{
			return false;
		}
		
	}	
	/**
	 * Return current authentication key
	 *
	 * @return string
	 */
	function GetAuthKey()
	{
		return $this->_auth_key;
	}
		
	/**
	 * Returns last error 
	 *
	 * @return string
	 */
	function GetErrors()
	{
		return ($this->_exception != null ? $this->_exception->getCode() . " : " . $this->_exception->getMessage() : '');
	}

	/**
	 * Returns last error code
	 *
	 * @return string
	 */
	function GetErrorCode()
	{
		return ($this->_exception != null ? $this->_exception->getCode() : '');
	}

	/**
	 * Returns last error message
	 *
	 * @return string
	 */
	function GetErrorMsg()
	{
		return ($this->_exception != null ?  $this->_exception->getMessage() : '');
	}

    /**
     * Execute function
     *
     * @param $method string
     * @param $params array of mixed
     * @return mixed
     */
    private function execute($method, $params = array())
    {
        $this->_exception = null;
        $result = null;
//print_r($params);
        $params['Method'] = $method;
        if (!isset($params['auth_key']) || !strlen($params['auth_key'])) {
            $params['auth_key'] = $this->_auth_key;
        }
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->_api_url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);

        $result = curl_exec($ch);

        if ($result === false) {
            unset($result);
            $this->_exception = curl_error($ch);
        }
        curl_close($ch);

        if (isset($result))
        {
            $json_object = json_decode($result, true);
            return $json_object['Result'];
        }

        return null;
    }

}

/* the end of file */